/*
* KiezelPay Integration Library - v2.1 - Copyright Kiezel 2016
*
* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO 
* WARRANTY FOR THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE 
* LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT 
* HOLDERS AND/OR OTHER PARTIES PROVIDE THE LIBRARY "AS IS" 
* WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, 
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE
* RISK AS TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU.
* SHOULD THE LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
* NECESSARY SERVICING, REPAIR OR CORRECTION.
* 
* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN 
* WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY 
* MODIFY AND/OR REDISTRIBUTE THE LIBRARY AS PERMITTED ABOVE, BE 
* LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, 
* INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR 
* INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS
* OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY 
* YOU OR THIRD PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH
* ANY OTHER SOFTWARE), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN 
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*/

#pragma once

#include <pebble.h>

/**
  All events that can be fired by kiezelpay lib.
  The event types are flags which can be combined in the output of the function kiezelpay_get_status()
*/
typedef enum {
  KIEZELPAY_NO_EVENT              = 0,
  KIEZELPAY_BLUETOOTH_UNAVAILABLE = 1 << 0,       /**< Cannot send appmessage to phone */
  KIEZELPAY_INTERNET_UNAVAILABLE  = 1 << 1,       /**< Cannot connect with kiezelpay server */
  KIEZELPAY_ERROR                 = 1 << 2,       /**< Generic error (e.g. invalid message format) */
  KIEZELPAY_TRIAL_STARTED         = 1 << 3,       /**< Trial has started (trial end time [uint32_t] as extra data) ==> unlock all features available during trial */
  KIEZELPAY_TRIAL_ENDED           = 1 << 4,       /**< Trial has ended, kiezelpay lib is attempting to initiate the purchase on the server ==> disable all trial features */
  KIEZELPAY_CODE_AVAILABLE        = 1 << 5,       /**< A purchase transaction has been initiated on the kiezelpay server (purchase code [uint32_t] as extra data) */
  KIEZELPAY_PURCHASE_STARTED      = 1 << 6,       /**< The user entered the code on the kiezelpay website and purchase is in progress (purchase code [uint32_t] as extra data) */
  KIEZELPAY_LICENSED              = 1 << 7        /**< User has successfully completed the purchase and is now licensed to use your app ==> unlock all features available after purchase */
} kiezelpay_event;

/** 
  Handler signature to catch kiezelpay events.
  Return true if you handled the event, return false to let the KiezelPay library handle events and show the correct messages to the user
*/
typedef bool (*kiezelpay_event_handler)(kiezelpay_event,void*);

/**
  Optionally set an event handler if you want to handle above events. Set to NULL to unregister.
*/
void kiezelpay_set_event_handler(kiezelpay_event_handler event_cb);

/** Call once on watchface/app init before using any other functions */
void kiezelpay_init();

/** Call on watchface/app deinit to clean up mess */
void kiezelpay_deinit();

/**
  Signal the kiezelpay lib that the user needs to purchase to continue. You need to call this when 
  there is no trial (KIEZELPAY_DISABLE_TIME_TRIAL == 1), you can call also it when you do have a trial
  in case you want to start the purchase process before the trial ends. If you dont call it in that case
  the purchase will start automatically as soon as the trial ends
*/
void kiezelpay_start_purchase();

/**
  Signal the kiezelpay lib that a purchase is no longer necesarry (e.g. the user backed out of the premium feature and does not want to use it anyway).
*/
void kiezelpay_cancel_purchase();

/**
  Get the time when the current trial will end. You can use this to inform the user how long his trial still lasts.
  This trial end time is also given along with the KIEZELPAY_TRIAL_STARTED event
*/
time_t kiezelpay_get_trial_end_time();

/**
  Get user status info ==> this info is also communicated with the on_kiezelpay_event handler, you can use these to get the status at any other time as well.
  The return value of this function can be any combination of the kiezelpay_event values!
  e.g. trial ended and no internet; result == KIEZELPAY_TRIAL_ENDED | KIEZELPAY_NO_INTERNET
   ==> test the values like "if (result & KIEZELPAY_TRIAL_ENDED) { }"
  BEWARE: this function can return the wrong result in the situation that a user deleted the app AFTER having purchased it, then later reinstalls the app. 
  In that case the library "thinks" this is a clean install and will NOT return the status "KIEZELPAY_LICENSED" until there was communication with the 
  kiezelpay server and it received the licensed status from the kiezelpay server (should not be more then a few seconds after installation). 
  Keep this in mind when showing messages to the user based on the result of this function.
*/
int32_t kiezelpay_get_status();

#define KIEZELPAY_BBM_DEFINED
/**
  The function below allows you to define how KiezelPay will handle the watch back button 
  when the KiezelPay purchase is in progress and the KiezelPay purchase dialog is visible.
  
  This will not affect the normal function of the back button in any way when the purchase
  dialog is not visible.
  
  These settings do not affect how the back button is handled in watchfaces, only in watchapps.
  
  DEFAULT: "KIEZELPAY_BACK_BTN_EXIT" ==> will exit the app when the back button is pressed
  
  WARNINGS: 
    - "KIEZELPAY_BACK_BTN_DEFAULT" will allow the user to close the purchase dialog
      in a watchapp (not in a watchface) which then allows the user to use the app without
      paying if you do not add extra checks for this scenario.
    - "KIEZELPAY_BACK_BTN_DISABLE" will lock the user in the purchase dialog unable
      to get out of your app in case of a watchapp (not in a watchface). This is a very
      bad user experience and will make your app look like ransomware!
*/
typedef enum {
  KIEZELPAY_BACK_BTN_DEFAULT,    /**< Leave default behaviour */
  KIEZELPAY_BACK_BTN_DISABLE,    /**< Disable the back button */
  KIEZELPAY_BACK_BTN_EXIT        /**< Exit the app or watchface (default setting) */
} kiezelpay_back_button_mode;

void kiezelpay_set_back_button_mode(kiezelpay_back_button_mode mode);

